package com.ashokit.ticketbooking.config;


import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.Customizer;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.provisioning.InMemoryUserDetailsManager;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

@Configuration
@EnableWebSecurity
public class SecurityConfig {
	
	
	//We will populate our Authentication Data and we will populate our authorization Rules
	
	
	//Authentication
	//UserDetails is the Spring boot Authentication Object
	
	@Bean
	public UserDetailsService userDetailsService(PasswordEncoder encoder) {
		
		//The following line create a new user ashokitadmin with password ashokitadmin belongs to ADMIN AND GUEST ROLE
		UserDetails admin=User.withUsername("ashokitadmin").
				password(encoder.encode("ashokitadmin")).roles("ADMIN","GUEST").build();
		
		//The following line create a new user ashokitguest with password ashokitguest belongs to  GUEST ROLE
		
		UserDetails user=User.withUsername("ashokitguest").
				password(encoder.encode("ashokitguest")).roles("GUEST").build();
		
		return new InMemoryUserDetailsManager(admin,user);
		
	}

	
	@Bean//This line i will explain in next class
	public PasswordEncoder passwordEncoder() {
		return new BCryptPasswordEncoder();  //Base64 Password Format
	}
	
	
	//Authorization Logic
	
	//Rule1 ==> Admin Role can access All Tickets and Individual Ticket
	//Rule2 ==> Guest Role can access only Individual Ticket
	
	
	@Bean
	public SecurityFilterChain securityFilterChain(HttpSecurity authObj) throws Exception {
		
		//For the fisttime we dont know the role ==> formLogin ==> Will throw login page
		
		
		//Rule1 ==> Admin Role can access All Tickets and Individual Ticket
		//authObj.csrf().disable() //This line i will explain in next class
		//.authorizeHttpRequests().requestMatchers("/tickets/all/**").hasAnyRole("ADMIN").and().httpBasic();
		
		//http://localhost:8080/tickets/all ==> All Ticket API
		//http://localhost:8080/tickets/ticket/1 ==> Individual Ticket api
		
		
	     return authObj.authorizeHttpRequests(request -> request.requestMatchers(new AntPathRequestMatcher("/tickets/all/**"))
	    		 .hasAnyRole("ADMIN"))
	            .authorizeHttpRequests(request -> request.requestMatchers(new AntPathRequestMatcher("/tickets/ticket/**"))
	                .hasRole("GUEST")
	                .anyRequest()
	                .authenticated())
	            .formLogin(Customizer.withDefaults())
	            .build();
					
			
	            /*
		  authObj.csrf(csrf -> csrf.disable())
  		      .authorizeHttpRequests(request -> request.requestMatchers(new AntPathRequestMatcher("/tickets/all/**"))
	                .hasAnyRole("ADMIN")
	                .anyRequest()
	                .authenticated()).formLogin();
		  
		  
		  
		  authObj.csrf(csrf -> csrf.disable())
		      .authorizeHttpRequests(request -> request.requestMatchers(new AntPathRequestMatcher("/tickets/ticket/**"))
                .hasAnyRole("GUEST")
                .anyRequest()
                .authenticated()).formLogin(); 
		  return authObj.build();
				*/
		//http://localhost:8080/tickets/ticket/1
		//Rule2 ==> Guest Role can access only Individual Ticket
		//authObj.csrf().disable() //This line i will explain in next class
		//.authorizeHttpRequests().requestMatchers("/tickets/ticket/**").hasAnyRole("ADMIN","GUEST").and().httpBasic();
		
		
		
		
	}
	
}
